// include the library code:
#include <LiquidCrystal.h>

// Выводы ардуино при использовании шилда
LiquidCrystal lcd(8, 9, 4, 5, 6, 7);

const int dt1 = 103;      //Постоянные собственной задержки.
const int dt2 = 105;
unsigned int val = 60000;
volatile boolean enabled = false;
volatile unsigned long delay_tics = 0;


void setup() {
  noInterrupts(); //Настройка таймера
  TCCR1A = 0x00;
  TCCR1B = 0x00;
  TCCR1C = 0x00;
  TIMSK1 = 0x01;
  interrupts();
  pinMode(A4, OUTPUT);    //Выход управления оптроном
  pinMode(2, INPUT);      //Вход сигнала с оптрона
  pinMode(3, OUTPUT);
  
  //Заставка перед началом работы
  lcd.begin(16, 2);
  lcd.setCursor(0, 0);
  lcd.print("Optocoupler test");
  lcd.setCursor(6, 1);
  lcd.print("2021");
  delay(1000);
  lcd.clear();

  lcd.setCursor(0, 0);
  lcd.print("input=");
}

void IntCapt () {       //Прерывание по сигналу с оптопары
  TCCR1B = 0x00;        //Стоп таймера
  digitalWrite(3, HIGH);
  delay_tics = TCNT1;   //Измеренное число тактов
  enabled = true;       //Разрешить дальнейшую работу
  digitalWrite(3,LOW);
}

ISR (TIMER1_OVF_vect) { //Прерывание по истечению времени (~4мс)
  TCCR1B = 0x00;        //Стоп таймера
  delay_tics = 64666;   //Максимальное значение обозначает переполнение, например, если ничего не подключено
  enabled = true;       //Разрешить дальнейшую работу
}

void GetMeasurement(boolean polar) {  //Проведение замера для заданной полярности (фронт/спад)
  TCCR1B = 0x00;        //Стоп таймера
  TCNT1 = 0x00;                          //Сброс таймера
  enabled = false;                    //Флаг ожидания конца измерения

  if (polar) {                        //Если полярность положительная (Off-On)
    digitalWrite(A4, LOW);            //Установка низкого уровня на выходе - Начальное состояние
    delayMicroseconds(100);           //Задержка 100мкс
    attachInterrupt(digitalPinToInterrupt(2), IntCapt, FALLING); //Разрешение прерывания по входу
    TCCR1B = 0x01;                    //Запуск таймера
    digitalWrite(A4, HIGH);           //Установка высокого уровня на выходе - Конечное состояние
    delayMicroseconds(100);           //Длительность активного импульса 100мкс
    digitalWrite(A4, LOW);            //Выключение выхода
  }  else {                           //Для отрицательной полярности (On-Off)
    digitalWrite(A4, HIGH);           //Устанавка высокого уровня на выходе - Начальное состояние
    delayMicroseconds(100);           //Задержка 100мкс
    attachInterrupt(digitalPinToInterrupt(2), IntCapt, RISING); //Разрешение прерывания по входу
    TCCR1B = 0x01;                    //Запуск таймера
    digitalWrite(A4, LOW);            //Установка низкого уровня на выходе - Конечное состояние
    delayMicroseconds(100);
    digitalWrite(A4, LOW);            //Выключение выхода
  }
}

void loop() {                       //Основной цикл

  int x = analogRead(A0);           //Считывание клавиш

  delay(50);

  if (x < 50) {                       //Right - Rising
    lcd.setCursor(0, 1);
    lcd.print("rising        ");
    GetMeasurement(true); //Get result
    while (!enabled); //Wait until measurement is finished
    detachInterrupt(digitalPinToInterrupt(2));
    val = (delay_tics - dt1)*125/2;   //delay in nS
    lcd.setCursor(7, 1);
    lcd.print(val);
    lcd.print(" ns");       
  }

  if ((x > 400) && (x < 650)) {       //LEFT - Fallingg
    lcd.setCursor(0, 1);
    lcd.print("faling          ");
    GetMeasurement(false); //Get result
    while (!enabled); //Wait until measurement is finished
    detachInterrupt(digitalPinToInterrupt(2));
    val = (delay_tics - dt2)*125/2;   //delay in nS
    lcd.setCursor(7, 1);
    lcd.print(val);
    lcd.print(" ns");   
  }

if (val > 30000) {
    lcd.setCursor(0, 0);
    lcd.print("        ");
    lcd.setCursor(0, 0);    
    lcd.print("Not found ");    
} else if (val > 1000) {
    lcd.setCursor(0, 0);
    lcd.print("        ");
    lcd.setCursor(0, 0);    
    lcd.print("slow-PC817");  
} else if (val >= 100) {
    lcd.setCursor(0, 0);
    lcd.print("        ");
    lcd.setCursor(0, 0);    
    lcd.print("midle-XXXX");  
} else if (val < 100) {
    lcd.setCursor(0, 0);
    lcd.print("        ");
    lcd.setCursor(0, 0);    
    lcd.print("fast-6N137");  
}
  x = 0;
}


